<TeXmacs|0.3.4.3>

<style|article>

<\body>
  <assign|code|<macro|body|<surround|<vspace*|1fn>|<vspace|1fn><format|no
  indentation after>|<with|font family|tt|language|verbatim|<arg|body>>>>><ap\
  ply|title|Background behind Mathemagix><apply|author|Joris van der
  Hoeven><apply|address|Laboratoire de Mathmatiques (bt. 425)<format|next
  line>Universit Paris-Sud<format|next line>91405 Orsay Cedex<format|next
  line>France>

  <apply|maketitle>

  <\abstract>
    In this paper, we describe some of the background behind Mathemagix,
    including its motivations, scope and design goals, and a comparison with
    other languages.
  </abstract>

  <section|Introduction>

  <subsection|Motivations for a new language>

  My wish for a new general purpose computer algebra language is motivated by
  two main reasons: the quasi-absense of free computer algebra systems and
  the non existence of sufficiently general compiled computer algebra
  languages.

  The importance of free computer algebra systems is best understood when
  considering the analogy between a mathematical theorem and a program. What
  if mathematicians should pay the inventor of a theorem in order to build
  another theorem on top of it? Would you accept someone claiming to have
  proved something, and wanting to get credit for this, but yet not allowing
  you to read his proof? I believe that for the well being of mathematical
  research, people should be able to freely experiment with computer algebra
  systems, and freely release corrected distributions or extensions to such
  systems. I also believe that a result based on computations with a system
  whose source code is not available should not be considered as a valid
  mathematical result.

  As to my second motivation, I observe that the most prominent current
  computer algebra systems are either based on interpreted languages or mere
  libraries of fast routines in a conventional language. Interpreted
  languages are bad for many reasons, but mainly they are slow and usually
  not type safe. Type safeness is important when writing large computer
  algebra systems, because it forces programmers to define clearly specified
  interfaces. However, the design of type-safe compiled computer algebra
  languages is complicated, because of the richness of mathematical language.
  Aldor is closest to a language we are seeking for, and we will discuss its
  advantages and disadvantages below. However, the Aldor system is not free
  and a stable version has not even been released.

  <subsection|Design goals and scope>

  Mathemagix is intended to be the first free, type safe, compiled and
  general purpose computer algebra language. We want the language to be
  object oriented, very strongly typed, with support for signatures,
  conditional exports, and interdependent tuples and functions. Yet, the user
  should have a high control over the machine representation of data types.
  Since the main application of the language is the development of free
  computer algebra systems, Mathemagix should have good support for
  ``programming in the large''. Furthermore, Mathemagix should also be a good
  conventional programming language, which might very well compete with
  Pascal, C, C++, Ada, Java, etc.

  Issues which currently fall beyond the scope of the language are
  interactivity and parallelism. By interactivity, we mean that the language
  is not intended to be used inside a computer algebra session or shell.
  Indeed, for such purposes the user would be hindered by the rigidity of a
  type-safe language. Nevertheless, an additional user interface might be
  developed later, which is both compatible with the underlying compiled
  language, but which is more user friendly. We notice that it should also be
  possible to write Mathemagix interpreters. Parallelism is another issue
  about which we did not think seriously until now. Nevertheless, a parallel
  version of Mathemagix might be developed later on.

  <section|An analysis of Mathemagix>

  In this section, we follow some of the criteria from [Programming language
  concepts, Carlo Ghezzi & Mehdi Jazayeri, John Waley & Sons], and some of
  our own criteria, in order to evaluate Mathemagix.

  <subsection|Writablitity>

  <subsubsection|Simplicity>

  The language is simple in the sense that it provides a limited number of
  basic constructs, which often ressemble similar constructs in other object
  oriented languages. However, all possible combinations of these constructs
  may be more difficult to master and much effort still has to be made in
  order to figure out the precise semantics of all such combinations.

  Mathemagix is also redundant up to a certain level, although we do not
  think it to be worse in this respect than languages like C or Ada. Below
  follows an incomplete list of some of the most important redundancies (or
  issues which may be felt to be redundant) in the language:

  <\enumerate>
    <item>Like in C, numbers can be increased or decreased using <with|font
    family|tt|++> and <with|font family|tt|-->.

    <item>The Ada-like mechanism for passing parameters by name is supported
    in combination with the usual mechanism and default initializers, but we
    might still suppress it.

    <item>The <with|font family|tt|Tuple>, <with|font family|tt|Cross> and
    <with|font family|tt|Array>, <with|font family|tt|Record> constructs are
    quite similar. Records can also be declared using <with|font
    family|tt|with intern>.

    <item>Exports of a class can either be implemented in the interface in
    the usual implementation part, or in a postponed implementation part.

    <item>Functions can either be declared like <with|font
    family|tt|f(x:Int):Int==x*x;> or like <with|font
    family|tt|f:Int-\<gtr\>Int == (x:Int) <format|no line break>:-\<gtr\>
    x*x;>
  </enumerate>

  Nevertheless, several efforts have been made in order to remove too
  superfluous redundancies and in order to make up uniform conventions. For
  instance, data fields (in records and the like) are always accesses through
  <with|font family|tt|.> (in C, both <with|font family|tt|.> and <with|font
  family|tt|-\<gtr\>> are used), for data lookup (in arrays and the like) we
  always use <with|font family|tt|[]>, and function application
  systematically uses <with|font family|tt|()>.

  <subsubsection|Expressiveness>

  Mathemagix should be an absolute champion in expressiveness. Very abstract
  data types can be declared using a powerful class inheritance system. These
  types can be passed to and returned by functions, together with instances
  of these types. Abstract control structures can be created using
  generators. Yet, several modifiers are provided to control the correctness
  and low level details.

  <subsubsection|Orthogonality>

  The language is quite orthogonal, although we already mentioned the need
  for a more detailed examination of all possible combinations of the
  languages primitives. Nevertheless, the meaning of most combinations is
  easily predicted, and only obviously incorrect or certain obscure, but
  rarely used combinations are prohibited.

  <subsubsection|Definiteness>

  A more precise language definition is under progress. This definition will
  describe the formal syntax, the semantics of the language. It will also
  suggest an implementation of a compiler.

  <subsection|Readability>

  <subsubsection|Choice of notations>

  We have tried as much as possible to stay close to a syntax which is
  compatible with usual mathematical notation. For instance, we avoided using
  <with|font family|tt|=> for assignment and <with|font family|tt|==> for
  equality tests. We also adopted the usual precedence rules for mathematical
  operators.

  <subsubsection|Syntax>

  We adopted a classical block oriented syntax. As in C, we use <with|font
  family|tt|{> and <with|font family|tt|}> in order to delimit blocks, but we
  avoided the use of ``exceptional syntax rules'', like <with|font
  family|tt|for (<with|font shape|italic|init>; <with|font
  shape|italic|cond>; <with|font shape|italic|step>) <with|font
  shape|italic|body>;>. We also chose to let the names of variables precede
  their classes in declarations. This convention facilitates looking up the
  declaration of a variable. For a similar reason, we prohibited implicit
  declarations of variables by assignments or in the <with|font
  family|tt|for> part of a loop.

  <subsubsection|Comments and documentation>

  Comments can either be specified using <with|font family|tt|//>, in which
  case the comment runs to the end of the line, or using <with|font
  family|tt|{* <with|font shape|italic|comment> *}>, in which case comments
  can be nested and run over several lines. The second form of comments can
  be prefixed by <with|font family|tt|help>, in which case the compiler can
  extract the comments from the source files in order to compile a manual or
  documented listings.

  <subsection|Reliability>

  <subsubsection|Architecture independence>

  <subsubsection|Exception handling>

  <subsubsection|Side effects>

  <subsubsection|Unsecure language constructs>

  <subsection|Speed>

  <subsubsection|Compilation speed>

  We have sticked to the idea of the <with|font family|tt|import> keyword and
  resisted to the temptation to provide templates or pattern matching inside
  Mathemagix. Although it may sometimes be uncomfortable for the user to
  perform importations by hand, we believe that our decision will make
  programs more reliable and easier and faster to compile.

  Actually, most of the imports can usually be made at the start of a file or
  class definition. Indeed, for most applications, the <with|font
  family|tt|import> keyword behaves like an improved <with|font
  family|tt|#include>. In fact, importations correspond to a real action,
  since they may modify the behaviour of the original functions in the new
  context. For instance, <with|font family|tt|inline> functions may be
  imported as <with|font family|tt|outline>. In order to relieve the
  programmer a bit, we might nevertheless consider the automatic importation
  of the domain and range classes of functions in their bodies, but even this
  is a debatable issue.

  <subsubsection|Inlining>

  <subsubsection|Direct classes and data on the heap>

  <subsubsection|Garbage collection>

  <subsection|Programming in the large>

  <section|An analysis of Aldor>

  <subsection|Positive aspects of Aldor>

  <\itemize>
    <item>The type safeness and power of the Category/Signature and
    Domain/Type based approach.

    <item>The idea of conditional exports.

    <item>The idea of post facto extensions.

    <item>The idea of the <with|font family|tt|import> construct, which
    avoids a complicated template based approach for generic data structures
    (like in C++) and eases the job of implementing a compiler.

    <item>The generator data type and the <with|font family|tt|generate> and
    <with|font family|tt|yield> constructs, which allow abstract flow
    control.

    <item>An elegant <with|font shape|italic|mathematical> syntax (contrary
    to C++, java, etc.).
  </itemize>

  <subsection|A criticism of Aldor>

  <\itemize>
    <item>Aldor is not free.

    <item>The absense of reference types (like <with|font family|tt|&> in
    C++). As a consequence, there is no analogue for C++ like methods
    associated to a data type.

    <item>The absense of automatic/implicit type conversion.

    <item>No real mechanism for type inheritance; the only real form of type
    inheritance is for derived classes which have the same representation
    type as the base class. As a consequence, the mechanism for data
    protection is also quite rudimentary (although consistent).

    <item>It is not clear what the final code will look like. Implementation
    details have been hidden to the user. Furthermore, several low level data
    structures can not be used efficiently at a higher level.

    <item>Although Aldor provides several powerful constructs, the manual is
    not precise on the effect of possible combinations of these constructs.
    For instance, may a class be extended only in the same block where it was
    defined, or also in, say a function of the the class itself? Yet another
    example: is the following code correct?

    <\code>
      R:Ring := Integer;

      import from R;

      x:R := 1;

      R := Polynomial (R);
    </code>

    <item>In Axiom, too many categories have been introduced. The number of
    categories have to be kept as small as possible and one should take more
    advantage of conditional imports. It seems that this lesson from Axiom
    has not been completely integrated in Aldor.
  </itemize>

  <subsection|Some comments of Andrey Grozin on Aldor>

  <\enumerate>
    <item>Two worst rudiments of C, <with|font family|tt|#include> and
    macros, were introduced into Aldor. It is too easy to abuse these
    facilities; a high level language must have a special mechanism for
    importing stuff from libraries, not just <with|font family|tt|#include>
    of some header files (something like IMPORT / DEFINITION MODULE mechanism
    of Modula2 is much better).

    <item><with|font family|tt|per> and <with|font family|tt|rep> are
    implemented via the macro <with|font family|tt|pretend>. <with|font
    family|tt|pretend> is unsafe, and should be allowed only in a few clearly
    marked places (like UNSAFE MODULE in Modula3); <with|font family|tt|per>
    and <with|font family|tt|rep> are safe, and abundant everywhere.

    <item>Basic arithmetic types are too 32-bit oriented, this is definitely
    wrong for a modern language. I'd prefer <with|font family|tt|Integer> for
    bignums (as it is in Aldor), and <with|font family|tt|Integer(n)> for
    n-byte fixed length integers, so that I could use <with|font
    family|tt|Integer 4> on IA-32 or <with|font family|tt|Integer 8> on Alpha
    (and why not use <with|font family|tt|Integer 8> on IA-32 too, via
    program emulation?). Similarly, <with|font family|tt|Float> should be
    parametrized by mantissa length.

    <item>More debatable things:

    <\expand>
      enumerate-alpha
    <|expand>
      <item>Iterators (like in Sather) can make loops simpler, safer and more
      powerful. A more uniform syntax can be invented for loops, lists with
      iterators, as well as for sums and products, like in REDUCE <with|font
      family|tt|for i:=1:10 sum 1/factorial(i)>.

      <item>Indentation-sensitive syntax is nicer than lots of C-isms
      <with|font family|tt|{ } ;> (like in Python and Haskell) (well, I can
      use <with|font family|tt|#pile>, of course).

      <item>Aldor already incorporates some ideas from Haskell; some more
      would be nice (procedure definitions with patterns, for example - they
      are in Axiom but not in Aldor).
    </expand>
  </enumerate>

  I agree with point 1 for <with|font family|tt|#include>. As indicated in
  [Programming language concepts], the distinction between small scale and
  large scale programming should be as clear as possible. In Mathemagix, this
  principle has been applied to its extreme limit, since all program
  interdependencies are controlled via dedicated project files. On the other
  hand, macros may still be quite useful, especially the ``structure
  respecting'' Aldor macros.

  Point 2 does no longer apply in Mathemagix, while point 3 has been solved.
  As to point 4a, Sather indeed provides a generalization of the concept of
  generators. However, code like

  <\code>
    while i\<less\>10 loop {

    \ \ <with|font shape|italic|blah_blah>;

    }
  </code>

  might be more readable than

  <\code>
    loop {

    \ \ while i\<less\>10;

    \ \ <with|font shape|italic|blah_blah>;

    }
  </code>

  The slight gain in generality might not compensate this loss in
  readability.

  On the other hand, one might even think of more general abstract control
  structures. Actually, one may interpret a function like a routine which
  passes a return point as an argument and an abstract Sather loop as a
  function which passes a return point, a break point and a continue point as
  arguments. Similarly, one may think of exception as passing handler
  pointers as arguments (even though this may be implemented in a better
  way). It might therefore be interesting to design a language with a general
  mechanism for passing labels, thereby unifying abstract loops and exception
  handling. Such an abstraction might be particularly interesting in the case
  of parallellism.

  I do not agree with indentation-sensitive syntax in 4b, since this
  presupposes the use of dedicated editors and makes parsing more
  troublesome. I do not agree with 4c either, because we do not want to make
  Mathemagix a mixture of languages with different underlying philosophies.
  Pattern matching belongs to logical programming or template based
  programming and would be a nightmare for the implementer of a Mathemagix
  compiler.
</body>

<\initial>
  <\collection>
    <associate|odd page margin|30mm>
    <associate|paragraph width|150mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|paragraph hyphenation|professional>
    <associate|reduction page bottom margin|15mm>
    <associate|page type|a4>
    <associate|reduction page left margin|25mm>
    <associate|even page margin|30mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\auxiliary>
  <\collection>
    <\associate>
      toc
    <|associate>
      <vspace*|1fn><with|font series|<quote|bold>|1
      Introduction><htab|5mm>?<vspace|0.5fn>

      1.1 Motivations for a new language<htab|5mm>?

      1.2 Design goals and scope<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|2 An analysis of
      Mathemagix><htab|5mm>?<vspace|0.5fn>

      2.1 Writablitity<htab|5mm>?

      <with|left margin|<quote|5mm>|2.1.1 Simplicity<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.1.2 Expressiveness<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.1.3 Orthogonality<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.1.4 Definiteness<htab|5mm>?>

      2.2 Readability<htab|5mm>?

      <with|left margin|<quote|5mm>|2.2.1 Choice of notations<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.2.2 Syntax<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.2.3 Comments and
      documentation<htab|5mm>?>

      2.3 Reliability<htab|5mm>?

      <with|left margin|<quote|5mm>|2.3.1 Architecture
      independence<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.3.2 Exception handling<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.3.3 Side effects<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.3.4 Unsecure language
      constructs<htab|5mm>?>

      2.4 Speed<htab|5mm>?

      <with|left margin|<quote|5mm>|2.4.1 Compilation speed<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.4.2 Inlining<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.4.3 Direct classes and data on the
      heap<htab|5mm>?>

      <with|left margin|<quote|5mm>|2.4.4 Garbage collection<htab|5mm>?>

      2.5 Programming in the large<htab|5mm>?

      <vspace*|1fn><with|font series|<quote|bold>|3 An analysis of
      Aldor><htab|5mm>?<vspace|0.5fn>

      3.1 Positive aspects of Aldor<htab|5mm>?

      3.2 A criticism of Aldor<htab|5mm>?

      3.3 Some comments of Andrey Grozin on Aldor<htab|5mm>?
    </associate>
  </collection>
</auxiliary>
