
/******************************************************************************
* MODULE     : compound.cc
* DESCRIPTION: Evaluation of compound expressions
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <xtable.hh>
#include <function.hh>
#include <converter.hh>
namespace mmx {

/******************************************************************************
* Implicit conversions for function application
* ----------------------------------------------------------------------------
* The conversion uses a rudimentary caching mechanism.
* However, this mechanism never frees cached values and
* should therefore be improved in the future.
* ----------------------------------------------------------------------------
* The routine 'apply_prog' returns a program which should be evaluated
* in order to perform the required conversions. The returned program
* necessitates additional auxiliary variables which are stored in
* symbols 0#, 1#, ... In particular, 0# contains the function and
* #1 the argument(s). The auxiliary variables are stored in the current
* environment. We might have to create a local sub-environment
* for each function application in the future. New auxiliary variables
* can be introduced in conv_lambda.cc.
******************************************************************************/

static converter cvt;
static table     cvt_cache;

static generic
apply_prog (generic fun_type, generic arg_type) {
  generic pair= list (fun_type, arg_type);
  if (!cvt_cache->contains (pair))
    cvt_cache (pair)= cvt->apply_prog (fun_type, arg_type);
  return cvt_cache [pair];
}

/******************************************************************************
* Function application
******************************************************************************/

void
evaluator_rep::kw_compound_1 () {
  // Input: (compound_1, val_first, ...)
  generic next= car (rem);
  if (gentype (next) == KEYWORD_TYPE) {
    cur= next;
    rem= cdr (rem);
    // Output: (keyw, ...)
    (this ->* (as_keyword (next)->fn)) ();
  }
  else {
    list orig= as_list (car (trace));
    cur= KW_COMPOUND_2;
    rem= cons (next, list (), cdr (orig), cdr (rem));
    // Output: (compound_2, first_arg, compound_2, (func), rem_args, ...)
    kw_compound_2 ();
  }
}

void
evaluator_rep::kw_compound_2 () {
  // Input: (compound_2, val_next, rev_vals_prev, rem_args, cont, ...)
  list vals_done= cons (car (rem), as_list (cadr (rem)));
  list rem_args = as_list (caddr (rem));
  if (nil (rem_args)) {
    vals_done   = reverse (vals_done);
    value   fun = as_value (car (vals_done));
    list    args= cdr (vals_done);
    generic arg = cons (MMX_TUPLE, get_values (args));
    generic type= cons (ID_CROSS, get_types (args));
    generic conv= apply_prog (fun->type, type);
    if (nil (conv)) {
      cout << "fun= " << fun << "\n";
      cout << "args= " << args << "\n";
      error (car (trace), "Invalid function application");
      return;
    }
    table xtra= table ();
    xtra (CONV_FUNCTION)= fun;
    xtra (CONV_ARGUMENT)= value (arg, type);
    env= xtable (env, xtra);
    cur= conv[0];
    rem= cons (KW_COMPOUND_3, cdddr (rem));
    // Output: (conv_prog, kw_compound_3, cont, ...)
  }
  else {
    cur= car (rem_args);
    rem= cons (KW_COMPOUND_2, vals_done, cdr (rem_args), cdddr (rem));
    // Output: (next_arg, compound_2, rev_vals_prev, rem_args, cont, ...)
  }
}

void
evaluator_rep::kw_compound_3 () {
  // Input: (compound_3, result, cont, ...)
  xtable xenv= as_xtable (env);
  env= xenv->base_t;
  cur= cadr (rem);
  rem= cons (car (rem), cddr (rem));
  // Output: (cont, result, ...)
  trace= cdr (trace);
}

} // namespace mmx
