
/******************************************************************************
* MODULE     : evaluate.cc
* DESCRIPTION: The main evaluation routines
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <evaluator.hh>
#include <single.hh>
#include <keyword.hh>
#include <error.hh>
namespace mmx {

/******************************************************************************
* The main loop
******************************************************************************/

generic
evaluator_rep::loop () {
  while (!nil (cur)) {
    //cout << "Trace " << trace << lf;
    //cout << "Evaluate " << N(trace) << ", " << cur << lf;
    //cout << "Evaluate " << cur << ", " << rem << lf;
    switch (gentype (cur)) {
    case STRING_TYPE: {
      generic tmp= cur;
      cur= car (rem);
      rem= cons (value (tmp, ID_STRING), cdr (rem));
      break;
    }
    case SYMBOL_TYPE:
      if (env->contains (cur)) {
	generic tmp= cur;
	cur= car (rem);
	rem= cons (env [tmp], cdr (rem));
	//cout << "  --> " << car (rem) << lf;
      }
      else {
	symbol s= as_symbol (cur);
	if (is_int (s)) {
	  cur= car (rem);
	  rem= cons (value (single (as_int (s)), ID_INT), cdr (rem));
	}
	else if (is_string (s)) {
	  cur= car (rem);
	  rem= cons (value (as_string (s), ID_STRING), cdr (rem));
	}
	else {
	  // cout << "env= " << env << lf;
	  error (cur, "Unknown identifier");
	}
      }
      break;
    case LIST_TYPE:
      trace= cons (cur, trace);
      rem  = cons (KW_COMPOUND_1, rem);
      cur  = car (cur);
      break;
    case SINGLE_TYPE: {
      generic tmp= cur;
      cur= car (rem);
      rem= cons (value (tmp, ID_INT), cdr (rem));
      break;
    }
    case VALUE_TYPE: {
      generic tmp= cur;
      cur= car (rem);
      rem= cons (tmp, cdr (rem));
      break;
    }
    case KEYWORD_TYPE:
      (this ->* (as_keyword (cur)->fn)) ();
      break;
    default:
      error (cur, "Syntax error");
    }
  }
  //cout << "env= " << env << lf;
  return car (rem);
}

/******************************************************************************
* Evaluating expressions
******************************************************************************/

generic
evaluator_rep::evaluate (generic g) {
  env (MISC_TODO)= list ();
  env (MISC_DONE)= list ();
  trace= list ();
  cur= g;
  rem= list (generic ());
  return loop ();
}

void
evaluator_rep::error (generic where, string message) {
  // cout << "trace= " << trace << "\n";
  generic err= std_error (message, where);
  cur= generic ();
  rem= cons (list (symbol ("errors!"), err), rem);
}

/******************************************************************************
* The user interface
******************************************************************************/

evaluator mmeval;

generic
mmx_evaluate (generic g) {
  return mmeval->evaluate (g);
}

} // namespace mmx
