
/******************************************************************************
* MODULE     : gentable.cc
* DESCRIPTION: Generic tables
* COPYRIGHT  : (C) 2000  Joris van der Hoeven
*******************************************************************************
* This software falls under the GNU general public license and comes WITHOUT
* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
* If you don't have this file, write to the Free Software Foundation, Inc.,
* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
******************************************************************************/

#include <gentable.hh>
#include <symbol.hh>
namespace mmx {

/******************************************************************************
* Default implementation of some required functions for generic structures
******************************************************************************/

bool
gentable_rep::Equal (generic g) {
  if (gentype (g) != Type ()) return false;
  return gentable (this) == as_gentable (g);
}

int
gentable_rep::Arity () {
  return N (entries ());
}

void
gentable_rep::Print (ostream& out) {
  out << gentable (this);
}

/******************************************************************************
* Default implementations of virtual routines for generic tables
******************************************************************************/

generic
gentable_rep::get_binding (generic key) {
  (void) key;
  fatal_error ("no get_binding function associated to gentable",
	       "gentable_rep::contains", "gentable.cc");
  exit (1);
}

void
gentable_rep::set_binding (generic binding) {
  (void) binding;
  fatal_error ("no set_binding function associated to gentable",
	       "gentable_rep::contains", "gentable.cc");
  exit (1);
}

void
gentable_rep::reset (generic key) {
  (void) key;
  fatal_error ("no reset function associated to gentable",
	       "gentable_rep::contains", "gentable.cc");
  exit (1);
}

list
gentable_rep::entries () {
  fatal_error ("no entries function associated to gentable",
	       "gentable_rep::contains", "gentable.cc");
  exit (1);
}

bool
gentable_rep::contains (generic key) {
  return !nil (get_binding (key));
}

generic
gentable_rep::read (generic key) {
  generic binding= get_binding (key);
  if (nil (binding)) return generic ();
  return binding[1];
}

generic&
gentable_rep::write (generic key) {
  generic binding= get_binding (key);
  if (nil (binding)) {
    binding= list (key, generic ());
    set_binding (binding);
  }
  return binding[1];
}

bool
gentable_rep::local_contains (generic key) {
  return contains (key);
}

generic
gentable_rep::local_read (generic key) {
  return read (key);
}

generic&
gentable_rep::local_write (generic key) {
  return write (key);
}

/******************************************************************************
* Routines for generic tables
******************************************************************************/

ostream&
operator << (ostream& out, gentable t) {
  out << "#table";
  if (t->contains (symbol ("begin!2"))) out << "<...>";
  else print_entries (out, t->entries ());
  return out;
}

bool
operator == (gentable t1, gentable t2) {
  if (gentype (t1) != gentype (t2)) return false;
  if (N(t1) != N(t2)) return false;
  list l= t1->entries ();
  while (!nil (l)) {
    generic pair= car (l);
    if (!t2->contains (pair[0])) return false;
    if (pair[1] != t2[pair[0]]) return false;
    l= cdr (l);
  }
  return true;
}

bool
operator != (gentable t1, gentable t2) {
  return !(t1==t2);
}

} // namespace mmx
